/**************************************************************************
* The following is to enable dumps from inside JS.  If I can get this
* working I can use this to debug the code.
**************************************************************************/
var debugboolean = true;
//setting pref for debug
if(debugboolean==true)
{
    const PREFS_CID      = "@mozilla.org/preferences;1";
    const PREFS_I_PREF   = "nsIPref";
    const PREF_STRING    = "browser.dom.window.dump.enabled";
    try {
	var Pref        = new Components.Constructor(PREFS_CID, PREFS_I_PREF);
	var pref        = new Pref( );
	pref.SetBoolPref(PREF_STRING, true);
    } catch(e) { 
	alert(e);
    }
}

// Some XPCOM constants we need to walk the bookmarks tree
// Commented out because they are already defined in chrome://communicator/content/bookmarks/bookmarks.js
// const RDF = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService(Components.interfaces.nsIRDFService);
// const RDFC = Components.classes['@mozilla.org/rdf/container;1'].createInstance(Components.interfaces.nsIRDFContainer);
// const RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService(Components.interfaces.nsIRDFContainerUtils);
// const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
// const NC_NS = "http://home.netscape.com/NC-rdf#";
// const BMDS = Components .classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].getService(Components.interfaces.nsIRDFDataSource); 
// const RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
const XPPREF = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);
const USER_AGENT = navigator.appCodeName + "/" + navigator.appVersion + " Habari Xenu News Aggregator 0.6.5 (http://habarixenu.mozdev.org/; thaths@mozdev.org)";


// Some global variables that are used across functions
var feedsTree;		// The tree used to display the feeds
var xmlhttp;		// A HTTP request object
var localPath;		// Path to feeds downloaded into a local file
var win;		// The Habari Xenu window
var updateInterval;	// How often to update
var progmtr;		// The progress meter
var fetchURL;		// The Feed's URL
var initialized = false;

// HTTP request results
const RESULT_OK = 0;
const RESULT_PARSE_ERROR = 1;
const RESULT_NOT_RSS = 2;
const RESULT_NOT_FOUND = 3;
const RESULT_NOT_AVAILABLE = 4;
const RESULT_ERROR_FAILURE = 5;



/**************************************************************************
* This function gets called when we need to open the settings editor
**************************************************************************/
function openSettingsEditor() {
    dump("openSettingsditor(): Opening Settings editor\n");

    // Open a new modal dialog showing the settingseditor xul as chrome
    // pass it a reference to the feedsTree
    var settingsURL = "chrome://habarixenu/content/settingseditor.xul";
    window.openDialog(settingsURL,"","chrome,modal, width=340, height=270", feedsTree);

    dump("openSettingsditor(): Finished opening Settings editor\n");
    return true;
}



/**************************************************************************
* This function gets called when we need to open the settings editor
**************************************************************************/
function openBookmarksEditor() {
    dump("openBookmarksEditor(): Opening Settings editor\n");

    // Open a new window showing the bookmark editor xul as chrome
    var settingsURL = "chrome://communicator/content/bookmarks/bookmarksManager.xul";
    window.open(settingsURL,"","chrome");

    dump("openBookmarksEditor(): Finished opening Bookmarks editor\n");
    return true;
}


/**************************************************************************
* Close the application
**************************************************************************/
function closeHabariXenu() {
    dump("closeHabariXenu(): Closing window\n");
    window.close();
}



/**************************************************************************
* Populate the tree from the Datasource. See above for how we get 'ds'
* This function gets called on the XUL window's onload event
**************************************************************************/
function buildTreeFromDataSource() {
    dump("buildTreeFromDataSource(): Beginning to build subscription tree from bookmarks\n");

    // Initialize win to point to the main window
    win = document.getElementsByTagName("window")[0];

    // Activate the progress meter to indicate work happening
    win.setAttribute("wait-cursor", "true");
    progmtr = document.getElementById("progmtr");
    progmtr.setAttribute("mode", "undetermined");

    // If we don't run the following only once, it goes into a loop
    // for some reason, buildTreeFromDataSource() gets called a few times
    // when the application is loaded
    if (! initialized) {
	dump("buildTreeFromDataSource(): Not initialized.  Initializing...\n");

	// Open the about.html file and read all its contents in
	var fu = new FileUtils();
	var aboutFile = new File(fu.chromeToPath('chrome://habarixenu/locale/about.html'));

	aboutFile.open('r');
	var aboutContent = aboutFile.read();
	aboutFile.close();

	// Write the contents into the browser element
	var tbrowser = document.getElementById("parsed-rss-feed");
	tbrowser.contentDocument.open();
	tbrowser.contentDocument.write(aboutContent);
	tbrowser.contentDocument.close();

	initialized = true;
    }

    // Initialize some stuff in chrome://communicator/content/bookmarks/bookmarks.js
    initServices();
    initBMService();

    // Set the feedsTree to point to the tree in which feeds are stored
    feedsTree = document.getElementById("feedsTree");

    var prefFolder;

    // Has user selected one folder as preferred feeds folder
    if(XPPREF.getPrefType("habarixenu.feeds_folder") == XPPREF.PREF_INVALID){
	// No preferred folder.  Set it to the top of bookmarks RDF
	dump("buildTreeFromDataSource(): No feeds folder preference.  Setting defaults.\n");
	prefFolder = "NC:BookmarksRoot";
    } else {
	// Set to the preferred folder
	prefFolder = XPPREF.getCharPref("habarixenu.feeds_folder").toString();
	dump("buildTreeFromDataSource(): Existing preference.  Setting to " + prefFolder + "\n");
    }

    if(XPPREF.getPrefType("habarixenu.update_interval") == XPPREF.PREF_INVALID){
	// No update interval.  Set it to 60 mins
	dump("buildTreeFromDataSource(): No update interval preference.  Setting defaults.\n");
	updateInterval = 60;
    } else {
	// Set to the user's preferences
	updateInterval = XPPREF.getIntPref("habarixenu.update_interval");
	dump("buildTreeFromDataSource(): Existing preference.  Update interval is " + updateInterval + "\n");
    }

    // Set the attribute on the tree and rebuild it
    feedsTree.setAttribute("ref", prefFolder);
    feedsTree.builder.rebuild();

    // Deactivate the progress meter to indicate work done
    progmtr.setAttribute("mode", "determined");
    progmtr.setAttribute("value", "0%");
    win.removeAttribute("wait-cursor");

    dump("buildTreeFromDataSource(): Finished rebuilding subscription tree from bookmarks.\n");
}



/**************************************************************************
* This function gets called when the user selects an item in the feeds tree
* Here we find the URL and last update time of the feed.
**************************************************************************/
function fetchFeed(bmEvent) {
    dump("fetchFeed(): Entered.\n");

    // A flag to indicate whether we downloaded a fresh local copy of a remote feed
    var freshDownload = false;

    // If nothing was selected we have nothing to do
    if (feedsTree.currentIndex < 0) {
	dump("fetchFeed(): Nothing selected.  Returning.\n");
	return;
    }

    // If folder was selected we have nothing to do
    fetchURL = feedsTree.view.getCellText(feedsTree.currentIndex, "feedlist-url");
    if (! fetchURL) {
	dump("fetchFeed(): Folder selected.  Returning.\n");
	return false;
    }

    // Activate the progress meter to indicate work happening
    progmtr = document.getElementById("progmtr");
    progmtr.setAttribute("mode", "undetermined");
    progmtr.setAttribute("value", "50%");
    win.setAttribute("wait-cursor", "true");

    // The URN of the bookmark item that has been selected
    var selectURN = feedsTree.builderView.getResourceAtIndex(feedsTree.currentIndex).Value;

    const rName = RDF.GetResource(WEB_NS + "LastVisitDate");
    var rSource = RDF.GetResource(selectURN);
    var node =  BMDS.GetTarget(rSource, rName, true);

    var lastVisited;
    try {
	// Get the LastVisitDate for this bookmark if it exists
	lastVisited = node.QueryInterface(Components.interfaces.nsIRDFDate).Value;
	lastVisited /= 1000;	// nsIRDFDate is in microseconds JS Date() deals with milliseconds
    } catch (e) {
	// LastVisitDate does not exist for this bookmark.  Probably a newly 
	// added bookmark?  Assume it was created just after the moment of creation
	lastVisited = new Date(1);  // One second since epoch
    }

    var lastUpdatedAt = new Date(lastVisited);
    dump("fetchFeed(): Feed was last updated at " + lastUpdatedAt + "\n");

    var currentTime = new Date();
    dump("fetchFeed(): Current time is " + currentTime + "\n");

    // 1000 milliseconds in every second
    // 60 seconds in every minute
    // user wants to update every updateInterval minutes
    var oneHour = 1000 * 60 * updateInterval;

    var interval = currentTime - lastUpdatedAt;
    dump("fetchFeed(): interval since last update is " + interval + " ms\n");

    localPath = absPathToLocal(fetchURL);

    // Has it been an hour since the last update?
    if (interval > oneHour) {
	dump("fetchFeed(): Feed is stale.  Needs update.\n");
	downloadFromURL(fetchURL, localPath);
	freshDownload = true;
    } else {
	dump("fetchFeed(): Feed is supposed to be fresh.  Let us see if we have a local copy.\n");

	// Do we need to download a fresh copy from the server?
	try {
	    var localFile = new File(localPath);
	    if (localFile.exists()) {
		dump("fetchFeed(): We still have a local copy of the feed.  Going to parse it.\n")
		parseFeed(fetchURL);
	    } else {
		dump("fetchFeed(): We don't have a local copy of the feed.  Will fetch again.\n")
		downloadFromURL(fetchURL, localPath);
		freshDownload = true;
	    }
	} catch (e) {
	    dump("fetchFeed(): Error occured trying to see if local feed file " + localPath + " exists.\n");
	    dump(e + "\n");
	}
    }

    if (freshDownload) {
	// Newly downloaded feed.  Set its LastVisitDate in the BMDS
	dump("fetchFeed(): This is a newly downloaded feed.  Setting LastVisitDate in bookmarks.\n");

	var currentmicroseconds = currentTime.valueOf() * 1000;
	var newNode = RDF.GetDateLiteral(currentmicroseconds);

	if (node) {
	    // A LastVisitDate exists for this bookmark.  Change it
	    try {
		dump("Called with " + rSource + "," + rName + "," + node + "," + newNode + "\n");
		BMDS.Change(rSource, rName, node, newNode);
		BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
	    } catch (e) {
		dump("fetchFeed(): Error changing LastVisitDate of bookmark: " + e + "\n");
	    }
	} else {
	    // A LastVisitDate does not exist for this bookmark.  Create it
	    try {
		dump("Called with " + rSource + "," + rName + "," + newNode + "\n");
		BMDS.Assert(rSource, rName, newNode, true);
		BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
	    } catch (e) {
		dump("fetchFeed(): Error changing LastVisitDate of bookmark: " + e + "\n");
	    }
	}

    }

    // Reset feed
    feed = new Object;

    dump("fetchFeed(): Finished. Exiting.\n");
    return false;   // Returning false because we want this event to bubble up
}



/**************************************************************************
* A utility function that takes a URl, converts into an md5 hash, appends
* it to a local feeds directory location and returns the resulting string
**************************************************************************/
function absPathToLocal(url) {
    dump("absPathToLocal(): Called with " + url + "\n");
    var fu = new FileUtils();
    var hash = hex_md5(url);

    var chromeFolder = fu.chromeToPath("chrome://habarixenu/content/feeds/");
    var fullPath = "file:" + chromeFolder + hash;

    dump("absPathToLocal(): Returning " + fullPath + "\n");
    return fullPath;
}



/**************************************************************************
* A utility function to download the latest feed from a URL and store it
* locally into localPath
**************************************************************************/
function downloadFromURL(url, localPath) {
    dump("downloadFromURL(): Called with: " + url + "," + localPath + "\n");

    xmlhttp = new XMLHttpRequest();

    // On the following events run the named functions
    xmlhttp.onload = xmlhttpLoaded;
    xmlhttp.onerror = xmlhttpError;
    xmlhttp.onreadystatechange = xmlhttpReadyStateChange;

    // Initiate the request
    try {
	xmlhttp.open("GET", url, true);	// NOTE: Calling synchronously
	xmlhttp.setRequestHeader("User-Agent", USER_AGENT);
	xmlhttp.overrideMimeType("text/xml");
    } catch (e) {
	dump(e);
	xmlhttpGetResult(RESULT_ERROR_FAILURE);
    }

    try {
	xmlhttp.send(null);
    } catch (e) {
	dump(e);
	xmlhttpGetResult(RESULT_ERROR_FAILURE);
    }
}

function xmlhttpError(e) {
    alert("HTTP Error " + e);
}

function xmlhttpReadyStateChange() {
    if (xmlhttp.readyState == 2) {
	try {
	    if (xmlhttp.status == 404) {
		xmlhttpGetResult(RESULT_NOT_FOUND);
	    }
	} catch (e) {
	    xmlhttpGetResult(RESULT_NOT_AVAILABLE);
	    return;
	}
    } else if (xmlhttp.readyState == 3) {}
}

function xmlhttpLoaded(e) {
    var localFile;
    try {
	localFile = new File(localPath);
    } catch (e) {
	dump("xmlhttpLoaded(): Error initializing file: " + e + "\n");
    }

    try {
	if (! localFile.exists()) {
	    dump("xmlhttpLoaded(): " + localPath + " does not exist. Creating\n");
	    localFile.create();
	}
    } catch (e) {
	dump("xmlhttpLoaded(): Error creating file " + localPath + " : " + e + "\n");
    }

    if(localFile.open('w')) { 
	dump("downloadFromURL(): Writing fetched data to " + localPath + "\n");
	localFile.write("" + xmlhttp.responseText);
    }
    localFile.close();
    xmlhttpGetResult(RESULT_OK);
}

function xmlhttpGetResult(aResultCode) {
    xmlhttp.abort();
    if (aResultCode == RESULT_OK) {
	// Everything OK.  Will do something here
	parseFeed();
    }
}



function formatURL(url, ois) {
    if (ois) {
	return "'' onlick=tbrowser.addTab(" + url + ")";
    } else {
	return "'" + url + "' target='new'";
    }
}



/**************************************************************************
* A function that takes the RSS that has been loaded into the hidden
* iframe, goes through it using the DOM and creates an HTML representation
* of the data in the RSS
**************************************************************************/
function parseFeed() {
    var feed = '';
    try {
	dump("parseFeed(): parsing " + localPath + "\n");
	feed = new FeedParser(localPath);
    } catch (e) {
	dump("parseFeed(): Error parsing feed: " + e + "\n");
    }

    var parsedContent = '';
    var openInSame;
    var target;
    var tbrowser = document.getElementById("parsed-rss-feed");

    if(XPPREF.getPrefType("habarixenu.open_in_same") == XPPREF.PREF_INVALID){
	// No update interval.  Set it to 60 mins
	dump("buildTreeFromDataSource(): No display preference.  Setting to default false.\n");
	openInSame = false;
    } else {
	// Set to the user's preferences
	openInSame = XPPREF.getBoolPref("habarixenu.open_in_same");
	dump("buildTreeFromDataSource(): Existing display preference is " + openInSame + "\n");
    }

    openInSame = false;
    target = (openInSame) ? "" : " target='_new'";

    if (feed.type) {
	if (feed.typeRDF) {
	    // RDF/RSS 1.0 Feeds
	    dump("parseFeed(): feed is RDF\n");

	    parsedContent = "<HTML><HEAD>\n";
	    var titleText = feed.channel.title;
	    var linkText = feed.channel.link;
	    var descText = feed.channel.description;
	    dump(titleText + "\n");

	    parsedContent += "<TITLE>News from " + titleText + "</TITLE>\n";
	    parsedContent += "<link rel='stylesheet' type='text/css' href='chrome://habarixenu/skin/display.css'>\n";
	    parsedContent += "</HEAD><BODY onload='return true;'>\n";
	    parsedContent += "<script type='application/x-javascript' src='chrome://habarixenu/content/display.js'/>\n";

	    // TODO must figure out images
	    parsedContent += "<div class='header'>\n";
	    parsedContent += "<b class='header-title' id='header-title'><a href='" + linkText + "'>" + titleText + "</a></b><br>\n";
	    parsedContent += descText + "<br>\n";
	    // parsedContent += "<a href='' onClick='return toggleDetail()' id='detailsBtn' style='text-decoration: none; color: blue'>+</a> Details<br>\n"
	    // parsedContent += "<div class='details' style='display: none'>\n";
	    // parsedContent += "<b>Feed URL</b>: " + url + "<br>\n";
	    // parsedContent += "<b>Feed Type</b>: RDF<br>\n";
	    // parsedContent += "</div>\n";
	    parsedContent += "</div>\n";
	    dump("parseFeed(): title: " + titleText + "\n");
	    dump("parseFeed(): description: " + descText + "\n");

	    for (var i = 0; i < feed.item.length; i++) {
		parsedContent += "<div class='item'>\n";
		if (feed.item[i].title && feed.item[i].link) {
		    parsedContent += "<b><a href='" + feed.item[i].link + "'" + target + ">" + feed.item[i].title + "</a></b><br>\n";
		} else if (feed.item_rdfabout[i] && feed.item[i].title) {
		    parsedContent += "<b><a href='" + feed.item_rdfabout[i] + "'" + target + "'>" + feed.item[i].title + "</a></b><br>\n";
		} else {
		    parsedContent += "<b><a href='" + feed.item[i].link + "'" + target + ">" + feed.item[i].link + "</a></b><br>\n";
		}

		if (feed.item[i].description) {
		    parsedContent += feed.item[i].description;
		}
		parsedContent += "</div>\n";
	    }
	} else if (feed.typeRSS) {
	    // RDF/RSS 1.0 Feeds
	    dump("parseFeed(): feed is RSS\n");

	    parsedContent = "<HTML><HEAD>\n";
	    var titleText = feed.channel.title;
	    var linkText = feed.channel.link;
	    var descText = feed.channel.description;
	    dump(titleText + "\n");

	    parsedContent += "<TITLE>News from " + titleText + "</TITLE>\n";
	    parsedContent += "<link rel='stylesheet' type='text/css' href='chrome://habarixenu/skin/display.css'>\n";
	    parsedContent += "</HEAD><BODY onload='return true;'>\n";
	    parsedContent += "<script type='application/x-javascript' src='chrome://habarixenu/content/display.js'/>\n";

	    // TODO must figure out images
	    parsedContent += "<div class='header'>\n";
	    parsedContent += "<b class='header-title' id='header-title'><a href='" + linkText + "'" + target + ">" + titleText + "</a></b><br>\n";
	    parsedContent += descText + "<br>\n";
	    // parsedContent += "<a href='' onClick='return toggleDetail()' id='detailsBtn' style='text-decoration: none; color: blue'>+</a> Details<br>\n"
	    // parsedContent += "<div class='details' style='display: none'>\n";
	    // parsedContent += "<b>Feed URL</b>: " + url + "<br>\n";
	    // parsedContent += "<b>Feed Type</b>: RDF<br>\n";
	    // parsedContent += "</div>\n";
	    parsedContent += "</div>\n";
	    dump("parseFeed(): title: " + titleText + "\n");
	    dump("parseFeed(): description: " + descText + "\n");

	    for (var i = 0; i < feed.channel.item.length; i++) {
		parsedContent += "<div class='item'>\n";

		parsedContent += "<b><a href='" + feed.channel.item[i].link + "'" + target + ">";
		if (feed.channel.item[i].title) {
		    parsedContent += feed.channel.item[i].title + "</a><br>\n";
		} else {
		    parsedContent += feed.channel.item[i].link + "</a><br>\n";
		}

		if (feed.channel.item[i].description) {
		    parsedContent += feed.channel.item[i].description;
		}
		parsedContent += "</div>\n";
	    }
	}
    } else {
	parsedContent = "<HTML><HEAD>\n";
	parsedContent += "<TITLE>Feed Type not Supported</TITLE>\n";
	parsedContent += "<link rel='stylesheet' type='text/css' href='chrome://habarixenu/skin/display.css'>\n";
	parsedContent += "</HEAD><BODY onload='return true;'>\n";
	parsedContent += "<div class='header'>\n";
	parsedContent += "<b class='header-title' id='header-title'>Feed Type not Supported</b><br>\n";
	parsedContent += "Sorry.  I can't figure out what type of feed this is.  It doesn't seem to be of the type RSS, RDF or Atom";
	parsedContent += "</div>\n";
    }
    parsedContent += "</BODY>\n</HTML>";


    // Open up the visible ibrowser and write the parsed content onto it
    var parsedDocument = window.frames[0].document;
    parsedDocument.body.innerHTML = parsedContent;

    // var tbrowser = document.getElementById("parsed-rss-feed");

    // tbrowser.addTab();
    // tbrowser.removeCurrentTab();
    // var browsers = tbrowser.browsers;
    // var newItem = browsers.length - 1;
    // var newTab = browsers[newItem];
    // var newTab = tbrowser.browsers[0];

    // tbrowser.contentDocument.open();
    // tbrowser.contentDocument.write(parsedContent);
    // tbrowser.contentDocument.close();
    // newTab.contentDocument.reload();

    // alert("Contain tabs " + browsers.length);
    // alert("Length is " + window.frames.length);

    // var displaybrowser = document.getElementById("parsed-rss-feed");
    // dump("Setting src of display area to null");
    // displaybrowser.setAttribute("src", "");

    // parsedDocument.open();
    // parsedDocument.write(parsedContent);
    // parsedDocument.close();

    // Switch to normal cursor
    // win.setAttribute("wait-cursor", "false");

    // Deactivate the progress meter to indicate work done
    progmtr.setAttribute("mode", "determined");
    progmtr.setAttribute("value", "0%");
    win.removeAttribute("wait-cursor");
    return true;
}



/**************************************************************************
* Show information about Habari Xenu
**************************************************************************/
function loadAbout() {
    var tbrowser = document.getElementById("parsed-rss-feed");
    var fu = new FileUtils();
    var aboutFile = new File(fu.chromeToPath('chrome://habarixenu/locale/about.html'));

    aboutFile.open('r');
    var aboutContent = aboutFile.read();
    aboutFile.close();

    tbrowser.contentDocument.open();
    tbrowser.contentDocument.write(aboutContent);
    tbrowser.contentDocument.close();

    return true;
}
