/**************************************************************************
* This file contains all the JavaScript needed by settings.xul.
* A lot of the ideas here are borrowed from the rssreader extension
**************************************************************************/
// Some XPCOM constants we need to walk the bookmarks tree
const RDF = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService(Components.interfaces.nsIRDFService);
const RDFC = Components.classes['@mozilla.org/rdf/container;1'].createInstance(Components.interfaces.nsIRDFContainer);
const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
const NC_NS = "http://home.netscape.com/NC-rdf#";
const BMDS = Components .classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].getService(Components.interfaces.nsIRDFDataSource); 
const RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService(Components.interfaces.nsIRDFContainerUtils);
const XPPREF = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);
var gNameArc;			    // Arc pointing to Name field in bookmarks
var selectedFolderId;		    // The ID of the folder selected by the user
var prefFolder;			    // The folder name as set in preferences
var prefUpdateInterval;		    // Update interval as set in preferences
var prefOpenInSame;		    // Display settings as set in preferences
var menuList;			    // The pulldown menulist in the dialog
var updateIntervalElement;	    // The text box containing the update interval
var openInSameElement;

// The main window's subscriptions tree passed to us as an argument
var bTree = window.arguments[0];

/**************************************************************************
* User has clicked OK.  Find out which item was selected, set it in
* prefs.js and return
* TODO must I reload the parent in some way to repaint?
**************************************************************************/
function doOK() {
    dump("doOK(): Entering\n");

    // Find out which element was selected in bookmarksFeedsFolderMenu
    menuList = document.getElementById("bookmarkFeedsFolder");
    updateIntervalElement = document.getElementById("update-interval");
    openInSameElement = document.getElementById("open-in-same");
    var menuSelectedElement = menuList.selectedIndex;
    var updateIntervalSet = updateIntervalElement.value;

    // Only if a feeds folder element selected
    if (menuSelectedElement >= 0) {
	dump("doOK(): Item number " + menuSelectedElement + " selected.  Setting preference\n");

	try {
	    XPPREF.setCharPref("habarixenu.feeds_folder", selectedFolderId);
	    XPPREF.setIntPref("habarixenu.update_interval", updateIntervalSet);
	    // XPPREF.setBoolPref("habarixenu.open_in_same", openInSameElement.checked);

	    bTree.setAttribute("ref", selectedFolderId);
	    bTree.builder.rebuild();
	} catch(e) {
	    alert(e);
	}
    } else {
	// TODO HAL: "I am sorry I can't do that, Dave."
	dump("doOK(): No folder was selected by user\n");
    }
    dump("doOK(): Leaving\n");
    return;
}


/**************************************************************************
* This function gets called when the user selects an item in the pulldown
* menu.  We set a variable with the ID of the selected element
**************************************************************************/
function selectBookmarkFolder(aEvent) {
    dump("selectBookmarkFolder(): Entering\n");

    selectedFolderId = aEvent.target.id;
    dump("selectBookmarkFolder(): Folder selected is " + selectedFolderId + "\n");

    dump("selectBookmarkFolder(): Leaving\n");
}


/**************************************************************************::
* User has clicked Cancel.
**************************************************************************/
function doCancel() {
    dump("doCancel(): Closing\n");
    return;
}


/**************************************************************************
* Populate the tree from the Datasource. See above for consts that we set
* This function gets called on the settings editor's onload event
**************************************************************************/
function buildMenuFromDataSource() {
    dump("buildMenuFromDataSource(): Beginning to build menu from RDF datasource\n");


    // Was any folder set as the preferred folder in prefs
    dump("buildMenuFromDataSource(): Getting user preferences\n");
    if(XPPREF.getPrefType("habarixenu.feeds_folder") == XPPREF.PREF_INVALID){
	prefFolder = "NC:BookmarksRoot";	// Nothing set.  Set it to the top of bookmarks RDF
    } else {
	prefFolder = XPPREF.getCharPref("habarixenu.feeds_folder").toString();
    }

    // Was any update interval set as the preferred in prefs
    if(XPPREF.getPrefType("habarixenu.update_interval") == XPPREF.PREF_INVALID){
	dump("buildMenuFromDataSource(): No update interval set. Setting to default of 60.\n");
	prefUpdateInterval = 60;		// Nothing set.  Set it to 60 minutes
    } else {
	prefUpdateInterval = XPPREF.getIntPref("habarixenu.update_interval");
	dump("buildMenuFromDataSource(): Preferred update interval is " + prefUpdateInterval + "\n");
    }

    // Was any update interval set as the preferred in prefs
    /*
    if(XPPREF.getPrefType("habarixenu.open_in_same") == XPPREF.PREF_INVALID){
	dump("buildMenuFromDataSource(): No pref for open_in_same set.  Assuming false\n");
	prefOpenInSame = false;		// Nothing set.  Set to false
    } else {
	prefOpenInSame = XPPREF.getBoolPref("habarixenu.open_in_same");
	dump("buildMenuFromDataSource(): Preferred display preference is " + prefOpenInSame + "\n");
    }
    */

    menuList = document.getElementById("bookmarkFeedsFolder");
    var menuPopup = document.getElementById("bookmarksFeedsFolderMenu");

    gNameArc = RDF.GetResource(NC_NS + "Name");

    // Remove any children menuPopup might have
    while (menuPopup.hasChildNodes()) {
	menuPopup.removeChild(menuPopup.firstChild);
    }

    var newMenuItem = document.createElementNS(XUL_NS, "menuitem");
    newMenuItem.setAttribute("label", "Bookmarks");
    newMenuItem.setAttribute("id", "NC:BookmarksRoot");
    menuPopup.appendChild(newMenuItem);

    var bookmarksRoot = RDF.GetResource("NC:BookmarksRoot");

    // Descend into this folder and read all its contents
    descendFolder(menuPopup, bookmarksRoot, 1, prefFolder);

    // Now set the update interval
    dump("buildMenuFromDataSource(): Setting prefUpdateInterval to " + prefUpdateInterval + "\n");
    updateIntervalElement = document.getElementById("update-interval");
    updateIntervalElement.value = prefUpdateInterval;

    // Check on uncheck the Display Preference
    /*
    dump("buildMenuFromDataSource(): Setting Display Preference to " + prefOpenInSame + "\n");
    openInSameElement = document.getElementById("open-in-same");
    openInSameElement.checked = prefOpenInSame;
    */

    /*
    if (prefOpenInSame) {
	openInSameElement.checked = true;
    } else {
	openInSameElement.checked = false;
    }
    */

    dump("buildMenuFromDataSource(): Finished rebuilding menu from RDF datasource.\n");
}


/**************************************************************************
* A recursive function that we will use to descend into each folder and
* subfolder of the bookmarks tree and create menuitems for each folder
**************************************************************************/
function descendFolder(aPopup, aFolder, aDepth) {
    RDFC.Init(BMDS, aFolder);

    var children = RDFC.GetElements();
    while (children.hasMoreElements()) {
	var curr = children.getNext();
	if (RDFCU.IsContainer(BMDS, curr)) {
	    curr = curr.QueryInterface(Components.interfaces.nsIRDFResource);
	    var element = document.createElementNS(XUL_NS, "menuitem");
	    var name = BMDS.GetTarget(curr, gNameArc, true).QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
	    var indentation = new Array(aDepth + 1).join("  ");
	    element.setAttribute("label", indentation + name);
	    element.setAttribute("id", curr.Value);
	    aPopup.appendChild(element);
	    if (curr.Value == prefFolder) {
		// This item is currently set in prefs as the preferred folder
		// select it
		menuList.selectedItem = element;
		selectedFolderId = prefFolder;
	    }
	    descendFolder(aPopup, curr, ++aDepth);
	    --aDepth;
	}
    }
}
